# Generated by Django 5.0.4 on 2024-05-01 14:34

import django.contrib.postgres.fields
import django_pg_rrule.operations
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        django_pg_rrule.operations.RruleExtension(),
        migrations.CreateModel(
            name="ExampleModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("rrule", models.TextField(blank=True, null=True)),
                (
                    "rdates",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.DateField(), blank=True, default=[], size=None
                    ),
                ),
                (
                    "rdatetimes",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.DateTimeField(),
                        blank=True,
                        default=[],
                        size=None,
                    ),
                ),
                (
                    "exdates",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.DateField(), blank=True, default=[], size=None
                    ),
                ),
                (
                    "exdatetimes",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.DateTimeField(),
                        blank=True,
                        default=[],
                        size=None,
                    ),
                ),
                ("datetime_start", models.DateTimeField(blank=True, null=True)),
                ("datetime_end", models.DateTimeField(blank=True, null=True)),
                ("date_start", models.DateField(blank=True, null=True)),
                ("date_end", models.DateField(blank=True, null=True)),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
